unit CFileHeader; //UTILISE PAR UPROPEDIT
{ce composant maintient une liste de proprits lues en dbut de fichier
  par les procedures LoadFromFile, LoadFromStream
si LoadFromFile est appel avec un paramtre string, il lit tout le fichier dans la string
  rcupre l'en-tte et retourne le reste du fichier dans la string
sinon, le reste du fichier est lu dans les procdures vnementielles OnLoad
si SaveToFile est appel avec un paramtre string, il insre l'en-tte en dbut de string
  et crit le rsultat dans le fichier
sinon, le reste du fichier est crit dans les procdures vnementielles OnSave
==> avec un paramtre string, OnLoad et OnSave ne sont pas appels

le rangement est au format texte (lignes spares par CRLF).
Compatibilit ascendante : si le fichier lu ne contient pas la ligne d'en-tte, TFileHeader est transparent
Contenu :
- en-tte et indicateur de version
- proprits, 1 par ligne, format <nom>=<valeur>
  la valeur ne peut contenir de CR ou de LF :
    SetValeur transforme tout CRLF, CR, LF en #7
    GetValeur transforme tout #7 en CRLF
- fin par une ligne blanche
procedures d'E/S :
    data reprsente des informations communiques  l'application appelente dans les evts OnLoad/OnSave
    procedure LoadFromFile(PF:string;const data:pointer=NIL);          charge les donnes et ferme le fichier
    procedure LoadFromFile(PF:string;var data:string);          charge les donnes et ferme le fichier
    procedure SaveToFile(PF:string;const data:pointer=NIL);            met  jour les donnes et ferme le fichier
    procedure SaveToFile(PF:string;const data:string);            met  jour les donnes et ferme le fichier
    procedure LoadFromStream(FS: TFileStream);  charge les donnes "au passage"
    procedure SaveToStream(FS: TFileStream);    range les donnes "au passage"
    procedure UpdateFile(PF:string);                       met  jour le header dans le fichier
    procedure Init;                             initialise FSLprop
- function GetValeur(nom:string):string;
- function NbValeurs:integer;
- procedure SetValeur(nom,valeur:string);       cre le nom si besoin, et renseigne la valeur
}
interface

uses
  SysUtils, Classes, U_CRC;

type
  TFHLSevent = procedure(FS: TFileStream; data:pointer) of object;
  TFileHeader = class(TComponent)
  private
    { Dclarations prives }
    FSLprop: TStringList;
    CRC: T_CRC;
    function CalCRC:T_CRC;
  protected
    { Dclarations protges }
    FOnLoad,FOnSave:TFHLSevent;

  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetNom(index:integer):string;
    function GetValeur(nom:string):string;
    procedure Init;
    procedure LoadFromFile(PF:string;const data:pointer=NIL);overload;
    procedure LoadFromFile(PF:string;var data:string);overload;
    procedure SaveToFile(PF:string;const data:pointer=NIL);overload;
    procedure SaveToFile(PF:string;const data:string);overload;
    procedure UpdateFile(PF:string);
    procedure LoadFromStream(FS: TFileStream;const data:pointer=NIL);
    procedure SaveToStream(FS: TFileStream;const data:pointer=NIL);
    function NbValeurs:integer;
    procedure SetValeur(nom,valeur:string);
    function Modified:boolean;
  published
    { Dclarations publies }
    property OnLoad: TFHLSevent read FOnLoad write FOnLoad;
    property OnSave: TFHLSevent read FOnSave write FOnSave;
    property Strings:TStringList read FSLprop;// write FSLprop;
    property Name;
    property Tag;
  end;

procedure Register;

implementation

uses StrUtils, DlphUtil, Dialogs, StringUtils;

const
  k_MarqueHeader = 'TFileHeader V1.00 030504 01234jphides43210';

constructor TFileHeader.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then FSLprop:= TStringList.Create;
end;

destructor TFileHeader.Destroy;
begin
  if not (csDesigning in ComponentState) then FSLprop.Free;
  inherited Destroy;
end;

function TFileHeader.CalCRC:T_CRC;
begin
  result:=CalculCRCStr(FSLprop.Text);
end;

function TFileHeader.GetNom(index:integer):string;
begin
  result:=FSLprop.Names[index];
end;

function TFileHeader.GetValeur(nom:string):string;
begin
  result:=FSLprop.Values[nom];
  result:=AnsiReplaceStr(result,#7,#13#10);
end;

procedure TFileHeader.Init;
begin
  FSLprop.Clear;
  CRC:=CalCRC;
end;

procedure TFileHeader.LoadFromFile(PF:string;const data:pointer=NIL);
var
  FS: TFileStream;
begin
  if not FileExists(PF) then exit;
  FS:= TFileStream.Create(PF,fmOpenRead or fmShareDenyNone);
  try
    LoadFromStream(FS,data);
  finally
    FS.Free;
  end;
end;

procedure TFileHeader.LoadFromFile(PF:string;var data:string);
var
  S: string;
begin
  if not FileExists(PF) then exit;
  data:=StringFromFile(PF);
  S:=LigExtraire(0,data);//ne modifie pas data
  if (S=k_MarqueHeader) then begin
    S:=LigLireEtOter(data);
    FSLprop.Clear;
    repeat
      S:=LigLireEtOter(data);
      if S='' then break;
      FSLprop.Add(S);
    until false;
    CRC:=CalCRC;
  end;
end;

function CheckPos(FS: TFileStream):boolean;
begin
  result:=FS.Position=0;
  if not result then ShowMessage('erreur TFileHeader : le flux n''est pas en dbut (Position>0)');
end;

procedure TFileHeader.LoadFromStream(FS: TFileStream;const data:pointer=NIL);
var
  S: string;
begin
  if not CheckPos(FS) then exit;
  S:=StreamReadLn(FS);
  if (S=k_MarqueHeader) then begin
    FSLprop.Clear;
    repeat
      S:=StreamReadLn(FS);
      if S='' then break;
      FSLprop.Add(S);
    until false;
    CRC:=CalCRC;
  end else FS.Position:=0;
  if Assigned(FOnLoad) then FOnLoad(FS,data);
end;

function TFileHeader.NbValeurs:integer;
begin
  result:=FSLprop.Count;
end;

procedure TFileHeader.SaveToFile(PF:string;const data:pointer=NIL);
var
  FS: TFileStream;
begin
  if FileExists(PF) then DeleteFile(PF);
  FS:= TFileStream.Create(PF,fmCreate or fmShareExclusive);
  try
    SaveToStream(FS,data);
  finally
    FS.Free;
  end;
end;

procedure TFileHeader.SaveToFile(PF:string;const data:string);
var
  i: integer;
  S: string;
begin
  S:=k_MarqueHeader+k_CRLF;
  for i:=1 to FSLprop.Count do S:=S+FSLprop[i-1]+k_CRLF;
  S:=S+k_CRLF; //une ligne blanche
  CRC:=CalCRC;
  StringToFile(S+data,PF);
end;

procedure TFileHeader.SaveToStream(FS: TFileStream;const data:pointer=NIL);
var
  i: integer;
begin
  if not CheckPos(FS) then exit;
  StreamWriteLn(FS,k_MarqueHeader);
  for i:=1 to FSLprop.Count do StreamWriteLn(FS,FSLprop[i-1]);
  StreamWriteLn(FS,'');
  CRC:=CalCRC;
  if Assigned(FOnSave) then FOnSave(FS,data);
end;

procedure TFileHeader.UpdateFile(PF:string);
var
  FS: TFileStream;
  buf: AnsiString;//8 bit par car
  FH: TFileHeader;
  n: Longint;
begin
  if not FileExists(PF) then exit;
  FS:= TFileStream.Create(PF,fmOpenRead or fmShareDenyNone);
  try
//limine le header
    FH:= TFileHeader.Create(self.Owner);
    try
      FH.LoadFromStream(FS,NIL);
    finally
      FH.Free;
    end;
//sauve le contenu
    n:=FS.Size-FS.Position;
    SetLength(buf,n);//6/6 supprim +1
    n:=FS.Read(buf[1],n);
  finally
    FS.Free;
  end;
  DeleteFile(PF);
  FS:= TFileStream.Create(PF,fmCreate or fmShareExclusive);
  try
    SaveToStream(FS,NIL);
    FS.Write(buf[1],n);
  finally
    FS.Free;
  end;
end;

procedure TFileHeader.SetValeur(nom,valeur:string);
var
  i: integer;
begin
  valeur:=AnsiReplaceStr(valeur,#13#10,#7); //d'abord CRLF
  valeur:=AnsiReplaceStr(valeur,#13,#7);
  valeur:=AnsiReplaceStr(valeur,#10,#7);
  i:=FSLprop.IndexOfName(nom);
  if i=-1
  then FSLprop.Add(nom+'='+valeur)
  else FSLprop.ValueFromIndex[i]:=valeur;
end;

function TFileHeader.Modified:boolean;
begin
  result := CalCRC <> CRC ;
end;

procedure Register;
begin
  RegisterComponents('Perso', [TFileHeader]);
end;

end.
